@@ -27,3 +27,10 @@ $texiMacro2c{'hypertarget'} = '';
 $htmlMacro2a{'hypertarget'} = '<a name="';
 $htmlMacro2b{'hypertarget'} = '">';
 $htmlMacro2c{'hypertarget'} = '</a>';
+
+#
+# Other
+#
+$manMacro{'authors'} = 'Jason Hickey \fIet. al.\fP';
+$texiMacro{'authors'} = 'Jason Hickey @emph{et. al.}';
+$htmlMacro{'authors'} = 'Jason Hickey <em>et. al.</em>';
