@@ -76,6 +76,8 @@ open Pos
  *    The login name of the user executing the process.
  * \subsection{HOME}
  *    The home directory of the user executing the process.
+ * \subsection{PID}
+ *    The OMake process id.
  * \end{doc}
  *)
 
@@ -1156,7 +1158,8 @@ let length_fun venv pos loc args =
  * \subsection{nth}
  *
  * \begin{verbatim}
- *    $(nth sequence) : value
+ *    $(nth i, sequence) : value
+ *       i : Int
  *       sequence : Sequence
  *    raises RuntimeException
  * \end{verbatim}
@@ -1180,6 +1183,120 @@ let nth_fun venv pos loc args =
        | _ ->
             raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 2, List.length args)))
 
+(*
+ * Get a subrange of a list.
+ *
+ * \begin{doc}
+ * \subsection{nth-hd}
+ *
+ * \begin{verbatim}
+ *    $(nth-hd i, sequence) : value
+ *       i : Int
+ *       sequence : Sequence
+ *    raises RuntimeException
+ * \end{verbatim}
+ *
+ * The \verb+nth-hd+ function returns the first \verb+i+ elements of
+ * the sequence.  An exception is raised if the sequence is not
+ * at least \verb+i+ elements long.
+ *
+ * For example, the expression \verb+$(nth-hd 2, a "b c" d)+ evaluates to \verb+a "b c"+.
+ *
+ * \subsection{nth-tl}
+ *
+ * \begin{verbatim}
+ *    $(nth-tl i, sequence) : value
+ *       i : Int
+ *       sequence : Sequence
+ *    raises RuntimeException
+ * \end{verbatim}
+ *
+ * The \verb+nth-tl+ function skips \verb+i+ elements of the sequence
+ * and returns the rest.  An exception is raised if the sequence is not
+ * at least \verb+i+ elements long.
+ *
+ * For example, the expression \verb+$(nth-tl 1, a "b c" d)+ evaluates to \verb+"b c" d+.
+ *
+ * \subsection{sub}
+ *
+ * \begin{verbatim}
+ *    $(sub off, len, sequent) : value
+ *       off : Int
+ *       len : Int
+ *       sequence : Sequence
+ *    raises RuntimeException
+ * \end{verbatim}
+ *
+ * The \verb+sub+ function returns a subrange of the sequence.
+ * Counting starts at 0.  An exception is raised if the specified
+ * range is not in bounds.
+ *
+ * For example, the expression \verb+$(sub 1, 2, a "b c" d e)+ evaluates to \verb+"b c" d+.
+ * \end{doc}
+ *)
+let rec nth_hd l_rev l i =
+   if i = 0 then
+      List.rev l_rev
+   else
+      match l with
+         h :: l ->
+            nth_hd (h :: l_rev) l (pred i)
+       | [] ->
+            raise (Invalid_argument "nth_hd")
+
+let rec nth_tl l i =
+   if i = 0 then
+      l
+   else
+      match l with
+         _ :: l ->
+            nth_tl l (pred i)
+       | [] ->
+            raise (Invalid_argument "nth_tl")
+
+let sub l off len =
+   nth_hd [] (nth_tl l off) len
+
+let nth_hd_fun venv pos loc args =
+   let pos = string_pos "nth-hd" pos in
+      match args with
+         [i; arg] ->
+            let i = int_of_value venv pos i in
+            let args = values_of_value venv pos arg in
+            let len = List.length args in
+               if i < 0 || i > len then
+                  raise (OmakeException (loc_pos loc pos, StringIntError ("index is out of bounds", i)));
+               ValArray (nth_hd [] args i)
+       | _ ->
+            raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 2, List.length args)))
+
+let nth_tl_fun venv pos loc args =
+   let pos = string_pos "nth-tl" pos in
+      match args with
+         [i; arg] ->
+            let i = int_of_value venv pos i in
+            let args = values_of_value venv pos arg in
+            let len = List.length args in
+               if i < 0 || i > len then
+                  raise (OmakeException (loc_pos loc pos, StringIntError ("index is out of bounds", i)));
+               ValArray (nth_tl args i)
+       | _ ->
+            raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 2, List.length args)))
+
+let subrange_fun venv pos loc args =
+   let pos = string_pos "subrange" pos in
+      match args with
+         [off; len; arg] ->
+            let off = int_of_value venv pos off in
+            let len = int_of_value venv pos len in
+            let args = values_of_value venv pos arg in
+            let alen = List.length args in
+               if off < 0 || len < 0 || off + len > alen then
+                  raise (OmakeException (loc_pos loc pos, StringIntError ("one or more indexes are out of bounds", off + len)));
+               ValArray (sub args off len)
+       | _ ->
+            raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 3, List.length args)))
+
 (*
  * Reverse a list.
  *
@@ -2131,6 +2248,7 @@ let () =
           "HOST",           (fun _ -> ValData Lm_uname.nodename);
           "OMAKE_VERSION",  (fun _ -> ValData Omake_magic.version);
           "USER",           (fun _ -> ValData user);
+          "PID",            (fun _ -> ValInt (Unix.getpid ()));
           "HOME",           (fun venv -> ValDir (venv_intern_dir venv home_dir))]
    in
    let builtin_funs =
@@ -2185,6 +2303,9 @@ let () =
        true,  "filter",                filter,              ArityExact 2;
        true,  "filter-out",            filter_out,          ArityExact 2;
        true,  "nth",                   nth_fun,             ArityExact 2;
+       true,  "nth-hd",                nth_hd_fun,          ArityExact 2;
+       true,  "nth-tl",                nth_tl_fun,          ArityExact 2;
+       true,  "subrange",              subrange_fun,        ArityExact 3;
        true,  "length",                length_fun,          ArityExact 1;
        true,  "rev",                   rev_fun,             ArityExact 1;
 
