@@ -57,7 +57,7 @@ open Pos
  * Utilities.
  *)
 let is_dir dir =
-   try (Unix.stat dir).Unix.st_kind = Unix.S_DIR with
+   try (Unix.LargeFile.stat dir).Unix.LargeFile.st_kind = Unix.S_DIR with
       Unix.Unix_error _ ->
          false
 
@@ -122,7 +122,7 @@ let dir venv pos loc args =
       match args with
          [arg] ->
             let values = values_of_value venv pos arg in
-            let values = List.map (node_value_of_value venv pos) values in
+            let values = List.map (dir_value_of_value venv pos) values in
                concat_array values
        | _ ->
             raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 1, List.length args)))
@@ -874,6 +874,7 @@ let glob venv pos loc args =
    let options = glob_options_of_env venv pos in
    let option = string_of_value venv pos option in
    let options = glob_options_of_string options option in
+   let options = Lm_glob.create_options options in
    let dirs = strings_of_value venv pos arg in
    let root = Dir.cwd () in
    let cwd = venv_dir venv in
@@ -941,6 +942,7 @@ let ls venv pos loc args =
    let ls_fun  = ls_fun_of_string option in
    let options = glob_options_of_env venv pos in
    let options = glob_options_of_string options option in
+   let options = Lm_glob.create_options options in
    let dirs = strings_of_value venv pos arg in
    let root = Dir.cwd () in
    let cwd  = Dir.name root (venv_dir venv) in
@@ -991,6 +993,7 @@ let subdirs venv pos loc args =
    in
    let options = glob_options_of_env venv pos in
    let options = glob_options_of_string options (string_of_value venv pos option) in
+   let options = Lm_glob.create_options options in
    let dirs = strings_of_value venv pos arg in
    let root = Dir.cwd () in
    let cwd = Dir.name root (venv_dir venv) in
@@ -1255,11 +1258,24 @@ let mkdir venv pos loc args =
  * itself.
  * \end{doc}
  *)
+
+(*
+ * XXX: JYH: HACK: if the file size is too large,
+ * represent it as a string.  We may want to make
+ * 64-bit ints a representable type.
+ *)
+let max_file_size = Int64.of_int max_int
+let clip_size i =
+   if i > max_file_size then
+      ValData (Int64.to_string i)
+   else
+      ValInt (Int64.to_int i)
+
 let create_stat_obj obj stat =
-   let obj = venv_add_field obj st_dev_sym (ValInt stat.Unix.st_dev) in
-   let obj = venv_add_field obj st_ino_sym (ValInt stat.Unix.st_ino) in
+   let obj = venv_add_field obj st_dev_sym (ValInt stat.Unix.LargeFile.st_dev) in
+   let obj = venv_add_field obj st_ino_sym (ValInt stat.Unix.LargeFile.st_ino) in
    let kind =
-      match stat.Unix.st_kind with
+      match stat.Unix.LargeFile.st_kind with
          Unix.S_REG -> "REG"
        | Unix.S_DIR -> "DIR"
        | Unix.S_CHR -> "CHR"
@@ -1269,15 +1285,15 @@ let create_stat_obj obj stat =
        | Unix.S_SOCK -> "SOCK"
    in
    let obj = venv_add_field obj st_kind_sym (ValString kind) in
-   let obj = venv_add_field obj st_perm_sym (ValInt stat.Unix.st_perm) in
-   let obj = venv_add_field obj st_nlink_sym (ValInt stat.Unix.st_nlink) in
-   let obj = venv_add_field obj st_uid_sym   (ValInt stat.Unix.st_uid) in
-   let obj = venv_add_field obj st_gid_sym   (ValInt stat.Unix.st_gid) in
-   let obj = venv_add_field obj st_rdev_sym  (ValInt stat.Unix.st_rdev) in
-   let obj = venv_add_field obj st_size_sym  (ValInt stat.Unix.st_size) in
-   let obj = venv_add_field obj st_atime_sym (ValFloat stat.Unix.st_atime) in
-   let obj = venv_add_field obj st_mtime_sym (ValFloat stat.Unix.st_mtime) in
-   let obj = venv_add_field obj st_ctime_sym (ValFloat stat.Unix.st_ctime) in
+   let obj = venv_add_field obj st_perm_sym (ValInt stat.Unix.LargeFile.st_perm) in
+   let obj = venv_add_field obj st_nlink_sym (ValInt stat.Unix.LargeFile.st_nlink) in
+   let obj = venv_add_field obj st_uid_sym   (ValInt stat.Unix.LargeFile.st_uid) in
+   let obj = venv_add_field obj st_gid_sym   (ValInt stat.Unix.LargeFile.st_gid) in
+   let obj = venv_add_field obj st_rdev_sym  (ValInt stat.Unix.LargeFile.st_rdev) in
+   let obj = venv_add_field obj st_size_sym  (clip_size stat.Unix.LargeFile.st_size) in
+   let obj = venv_add_field obj st_atime_sym (ValFloat stat.Unix.LargeFile.st_atime) in
+   let obj = venv_add_field obj st_mtime_sym (ValFloat stat.Unix.LargeFile.st_mtime) in
+   let obj = venv_add_field obj st_ctime_sym (ValFloat stat.Unix.LargeFile.st_ctime) in
       ValObject obj
 
 let stat_aux stat_fun venv pos loc args =
@@ -1310,13 +1326,13 @@ let stat_aux stat_fun venv pos loc args =
        | _ ->
             raise (OmakeException (loc_pos loc pos, ArityMismatch (ArityExact 1, List.length args)))
 
-let stat = stat_aux Unix.stat
+let stat = stat_aux Unix.LargeFile.stat
 
 let lstat =
    if Sys.os_type = "Win32" then
-      stat_aux Unix.stat
+      stat_aux Unix.LargeFile.stat
    else
-      stat_aux Unix.lstat
+      stat_aux Unix.LargeFile.lstat
 
 (************************************************************************
  * Links.
@@ -1677,7 +1693,7 @@ let mv_aux mv venv pos loc args =
  *)
 let cp_file info file1 file2 =
    if mv_prompt info file1 file2 then
-      let mode = (Unix.stat file1).Unix.st_perm in
+      let mode = (Unix.LargeFile.stat file1).Unix.LargeFile.st_perm in
       let mode = mode_of_string mode info.mv_mode in
       let fdr = Lm_unix_util.openfile file1 [Unix.O_RDONLY] 0 in
       let () =
@@ -1929,7 +1945,7 @@ let chmod_usage = "Create a directory"
 let chmod info filename =
    if info.chmod_verbose then
       printf "Changing permissions on %s@." filename;
-   let mode = (Unix.stat filename).Unix.st_perm in
+   let mode = (Unix.LargeFile.stat filename).Unix.LargeFile.st_perm in
    let mode = mode_of_string mode info.chmod_mode in
    if info.chmod_force then
       try Unix.chmod filename mode with
@@ -2152,7 +2168,7 @@ let search_path_aux fail venv pos loc args =
             (* List the path *)
             let cache = venv_cache venv in
             let path = values_of_value venv pos dirs in
-            let path = List.map (dir_of_value venv pos) path in
+            let path = Omake_eval.path_of_values venv pos path "." in
             let listing = Omake_cache.ls_path cache path in
 
             (* Find each file *)
@@ -2210,7 +2226,7 @@ let digest_path_aux fail venv pos loc args =
             (* List the path *)
             let cache = venv_cache venv in
             let path = values_of_value venv pos dirs in
-            let path = List.map (dir_of_value venv pos) path in
+            let path = Omake_eval.path_of_values venv pos path "." in
             let listing = Omake_cache.ls_path cache path in
 
             (* Find each file *)
@@ -2327,7 +2343,8 @@ let vmount_touch_files venv pos src dst =
    let _pos = string_pos "vmount_touch_files" pos in
    let src_name = Dir.fullname src in
    let dst_name = Dir.name src dst in
-   let _, files = list_dirs_rec [GlobIgnore [dst_name]] src_name ["."] in
+   let options = Lm_glob.create_options [GlobIgnore [dst_name]] in
+   let _, files = list_dirs_rec options src_name ["."] in
       List.iter (fun name -> ignore (venv_intern_cd venv PhonyProhibited dst name)) files
 
 (*
