@@ -195,8 +195,8 @@ let dir_test name =
 let cd_test venv pos loc dir =
    let name = Dir.fullname dir in
       try
-         let stat = Unix.stat name in
-            if stat.Unix.st_kind <> Unix.S_DIR then
+         let stat = Unix.LargeFile.stat name in
+            if stat.Unix.LargeFile.st_kind <> Unix.S_DIR then
                CdFile
             else if dir_test name then
                CdSuccess
