@@ -232,7 +232,7 @@ let stat_function venv pos file f =
    let pos = string_pos "stat_function" pos in
    let file = filename_of_string venv pos file in
    let stat =
-      try Some (Unix.stat file) with
+      try Some (Unix.LargeFile.stat file) with
          Unix.Unix_error _ ->
             None
    in
@@ -251,93 +251,93 @@ let file_exists stat =
 
 let is_block_special stat =
    match stat with
-      Some { Unix.st_kind = Unix.S_BLK } ->
+      Some { Unix.LargeFile.st_kind = Unix.S_BLK } ->
          true
     | _ ->
          false
 
 let is_char_special stat =
    match stat with
-      Some { Unix.st_kind = Unix.S_CHR } ->
+      Some { Unix.LargeFile.st_kind = Unix.S_CHR } ->
          true
     | _ ->
          false
 
 let is_dir stat =
    match stat with
-      Some { Unix.st_kind = Unix.S_DIR } ->
+      Some { Unix.LargeFile.st_kind = Unix.S_DIR } ->
          true
     | _ ->
          false
 
 let is_reg_file stat =
     match stat with
-      Some { Unix.st_kind = Unix.S_REG } ->
+      Some { Unix.LargeFile.st_kind = Unix.S_REG } ->
          true
     | _ ->
          false
 
 let is_symlink_file stat =
      match stat with
-      Some { Unix.st_kind = Unix.S_LNK } ->
+      Some { Unix.LargeFile.st_kind = Unix.S_LNK } ->
          true
     | _ ->
          false
 
 let is_socket_file stat =
      match stat with
-      Some { Unix.st_kind = Unix.S_SOCK } ->
+      Some { Unix.LargeFile.st_kind = Unix.S_SOCK } ->
          true
     | _ ->
          false
 
 let is_named_pipe_file stat =
      match stat with
-      Some { Unix.st_kind = Unix.S_FIFO } ->
+      Some { Unix.LargeFile.st_kind = Unix.S_FIFO } ->
          true
     | _ ->
          false
 
 let is_sticky_file stat =
    match stat with
-      Some { Unix.st_perm = perm } ->
+      Some { Unix.LargeFile.st_perm = perm } ->
          perm land 0o1000 <> 0
     | None ->
          false
 
 let is_setgid_file stat =
    match stat with
-      Some { Unix.st_perm = perm } ->
+      Some { Unix.LargeFile.st_perm = perm } ->
          perm land 0o2000 <> 0
     | None ->
          false
 
 let is_setuid_file stat =
    match stat with
-      Some { Unix.st_perm = perm } ->
+      Some { Unix.LargeFile.st_perm = perm } ->
          perm land 0o4000 <> 0
     | None ->
          false
 
 let is_group_owner_file stat =
    match stat with
-      Some { Unix.st_gid = gid } ->
+      Some { Unix.LargeFile.st_gid = gid } ->
          gid = Unix.getegid ()
     | None ->
          false
 
 let is_owner_file stat =
    match stat with
-      Some { Unix.st_uid = uid } ->
+      Some { Unix.LargeFile.st_uid = uid } ->
          uid = Unix.geteuid ()
     | None ->
          false
 
 let is_readable_file stat =
    match stat with
-      Some { Unix.st_uid = uid;
-             Unix.st_gid = gid;
-             Unix.st_perm = perm
+      Some { Unix.LargeFile.st_uid = uid;
+             Unix.LargeFile.st_gid = gid;
+             Unix.LargeFile.st_perm = perm
       } ->
          let my_uid = Unix.geteuid () in
          let my_gid = Unix.getegid () in
@@ -349,9 +349,9 @@ let is_readable_file stat =
 
 let is_writable_file stat =
    match stat with
-      Some { Unix.st_uid = uid;
-             Unix.st_gid = gid;
-             Unix.st_perm = perm
+      Some { Unix.LargeFile.st_uid = uid;
+             Unix.LargeFile.st_gid = gid;
+             Unix.LargeFile.st_perm = perm
       } ->
          let my_uid = Unix.geteuid () in
          let my_gid = Unix.getegid () in
@@ -363,9 +363,9 @@ let is_writable_file stat =
 
 let is_executable_file stat =
    match stat with
-      Some { Unix.st_uid = uid;
-             Unix.st_gid = gid;
-             Unix.st_perm = perm
+      Some { Unix.LargeFile.st_uid = uid;
+             Unix.LargeFile.st_gid = gid;
+             Unix.LargeFile.st_perm = perm
       } ->
          let my_uid = Unix.geteuid () in
          let my_gid = Unix.getegid () in
@@ -377,8 +377,8 @@ let is_executable_file stat =
 
 let is_nonempty_file stat =
    match stat with
-      Some { Unix.st_size = size } ->
-         size <> 0
+      Some { Unix.LargeFile.st_size = size } ->
+         size <> Int64.zero
     | None ->
          false
 
@@ -396,22 +396,22 @@ let eq_file =
             let pos = string_pos "eq_file" pos in
                stat_function2 venv pos file1 file2 (fun stat1 stat2 ->
                      match stat1, stat2 with
-                        Some { Unix.st_dev = dev1; Unix.st_ino = ino1 },
-                        Some { Unix.st_dev = dev2; Unix.st_ino = ino2 } ->
+                        Some { Unix.LargeFile.st_dev = dev1; Unix.LargeFile.st_ino = ino1 },
+                        Some { Unix.LargeFile.st_dev = dev2; Unix.LargeFile.st_ino = ino2 } ->
                            dev1 = dev2 && ino1 = ino2
                       | _ ->
                            false))
 
 let newer_than_file stat1 stat2 =
    match stat1, stat2 with
-      Some { Unix.st_mtime = mtime1 }, Some { Unix.st_mtime = mtime2 } ->
+      Some { Unix.LargeFile.st_mtime = mtime1 }, Some { Unix.LargeFile.st_mtime = mtime2 } ->
          mtime1 > mtime2
     | _ ->
          false
 
 let older_than_file stat1 stat2 =
    match stat1, stat2 with
-      Some { Unix.st_mtime = mtime1 }, Some { Unix.st_mtime = mtime2 } ->
+      Some { Unix.LargeFile.st_mtime = mtime1 }, Some { Unix.LargeFile.st_mtime = mtime2 } ->
          mtime1 < mtime2
     | _ ->
          false
@@ -584,6 +584,8 @@ let parse_intop op i right =
  *    unop string
  *    string
  *)
+let no_glob_options = Lm_glob.create_options []
+
 let rec parse_term venv pos tokens =
    let pos = string_pos "parse_term" pos in
       match tokens with
@@ -611,7 +613,7 @@ let rec parse_term venv pos tokens =
        | TokUnop (op, _) :: arg :: tokens ->
             UnopExp (op, arg), tokens
        | TokName _ :: arg :: tokens ->
-            MatchExp (regex_of_shell_pattern [] (string_of_token arg)), tokens
+            MatchExp (regex_of_shell_pattern no_glob_options (string_of_token arg)), tokens
        | arg :: tokens ->
             UnopExp (IsNonEmptyStringOp, arg), tokens
        | [] ->
