@@ -41,6 +41,7 @@
 #include <tlhelp32.h>
 #include <winsock.h>
 
+#include "unixsupport.h"
 #include "lm_compat_win32.h"
 
 #define Val_nil         (Val_int(0))
@@ -75,23 +76,6 @@
  */
 #define MAX_THREAD_COUNT        1024
 
-/*
- * HACK: this gets the handle from a file_descr.
- * This depends on the OCaml implementation, but
- * it is unlikely to change.
- */
-struct filedescr {
-  union {
-    HANDLE handle;
-    SOCKET socket;
-  } fd;
-  enum { KIND_HANDLE, KIND_SOCKET } kind;
-};
-
-#define Handle_val(v) (((struct filedescr *) Data_custom_val(v))->fd.handle)
-#define Socket_val(v) (((struct filedescr *) Data_custom_val(v))->fd.socket)
-#define Descr_kind_val(v) (((struct filedescr *) Data_custom_val(v))->kind)
-
 /*
  * Processes and groups.
  *
@@ -550,6 +534,7 @@ static void terminate_and_exit(void)
  */
 value omake_shell_sys_set_tty_pgrp(value v_pgrp)
 {
+    CAMLparam1(v_pgrp);
     int pgrp;
 
 #ifdef OSH_DEBUG
@@ -561,7 +546,7 @@ value omake_shell_sys_set_tty_pgrp(value v_pgrp)
     if(pgrp == 0)
         pgrp = INIT_PID;
     state->current_pgrp = pgrp;
-    return Val_unit;
+    CAMLreturn(Val_unit);
 }
 
 /*
@@ -569,35 +554,38 @@ value omake_shell_sys_set_tty_pgrp(value v_pgrp)
  */
 value omake_shell_sys_suspend(value v_pgrp)
 {
+    CAMLparam1(v_pgrp);
 #ifdef OSH_DEBUG
     fprintf(stderr, "omake_shell_sys_suspend\n");
     fflush(stderr);
 #endif
     if(process_group_map(suspend_process, Int_val(v_pgrp)) < 0)
         failwith("suspend");
-    return Val_unit;
+    CAMLreturn(Val_unit);
 }
 
 value omake_shell_sys_resume(value v_pgrp)
 {
+    CAMLparam1(v_pgrp);
 #ifdef OSH_DEBUG
     fprintf(stderr, "omake_shell_sys_resume\n");
     fflush(stderr);
 #endif
     if(process_group_map(resume_process, Int_val(v_pgrp)) < 0)
         failwith("suspend");
-    return Val_unit;
+    CAMLreturn(Val_unit);
 }
 
 value omake_shell_sys_kill(value v_pgrp)
 {
+    CAMLparam1(v_pgrp);
 #ifdef OSH_DEBUG
     fprintf(stderr, "omake_shell_sys_kill\n");
     fflush(stderr);
 #endif
     if(process_group_map(kill_process, Int_val(v_pgrp)) < 0)
         failwith("suspend");
-    return Val_unit;
+    CAMLreturn(Val_unit);
 }
 
 /*
@@ -606,6 +594,7 @@ value omake_shell_sys_kill(value v_pgrp)
  */
 value omake_shell_sys_create_thread_pid(value v_pgrp)
 {
+    CAMLparam1(v_pgrp);
     Process *processp;
     HANDLE event;
     int pgrp;
@@ -641,7 +630,7 @@ value omake_shell_sys_create_thread_pid(value v_pgrp)
     processp->wid = 0;
     processp->next = state->processes;
     state->processes = processp;
-    return Val_int(pid);
+    CAMLreturn(Val_int(pid));
 }
 
 /*
@@ -649,6 +638,7 @@ value omake_shell_sys_create_thread_pid(value v_pgrp)
  */
 value omake_shell_sys_init_thread_pid(value v_pid)
 {
+    CAMLparam1(v_pid);
     Process *processp;
     int pid;
 
@@ -668,7 +658,7 @@ value omake_shell_sys_init_thread_pid(value v_pid)
 
     /* Process has terminated */
     processp->thread = GetCurrentThreadId();
-    return Val_unit;
+    CAMLreturn(Val_unit);
 }
 
 /*
@@ -678,6 +668,7 @@ value omake_shell_sys_init_thread_pid(value v_pid)
  */
 value omake_shell_sys_release_thread_pid(value v_pid, value v_code)
 {
+    CAMLparam2(v_pid, v_code);
     Process *processp;
     int pid;
 
@@ -709,7 +700,7 @@ value omake_shell_sys_release_thread_pid(value v_pid, value v_code)
     fprintf(stderr, "omake_shell_sys_release_thread_pid: pid = %d\n", processp->pid);
     fflush(stderr);
 #endif
-    return Val_unit;
+    CAMLreturn(Val_unit);
 }
 
 /*
@@ -717,6 +708,7 @@ value omake_shell_sys_release_thread_pid(value v_pid, value v_code)
  */
 value omake_shell_sys_check_thread(value v_unit)
 {
+    CAMLparam1(v_unit);
     Process *processp;
     int id;
 
@@ -732,9 +724,9 @@ value omake_shell_sys_check_thread(value v_unit)
             break;
     }
     if(processp == 0)
-        return Val_int(0);
+        CAMLreturn(Val_int(0));
 
-    return Val_int(processp->killed);
+    CAMLreturn(Val_int(processp->killed));
 }
 
 /*
@@ -1067,6 +1059,7 @@ static BOOL WINAPI console_ctrl_handler(DWORD code)
  */
 value omake_shell_sys_init(value v_unit)
 {
+    CAMLparam1(v_unit);
     Process *processp;
     HANDLE c_stdin;
     DWORD mode;
@@ -1103,14 +1096,14 @@ value omake_shell_sys_init(value v_unit)
     /* Try to get the console */
     c_stdin = GetStdHandle(STD_INPUT_HANDLE);
     if(c_stdin == INVALID_HANDLE_VALUE)
-        return Val_unit;
+        CAMLreturn(Val_unit);
     status = GetConsoleMode(c_stdin, &mode);
     if(status)
         state->console = c_stdin;
 
     /* Install the console control handler */
     SetConsoleCtrlHandler(console_ctrl_handler, TRUE);
-    return Val_unit;
+    CAMLreturn(Val_unit);
 }
 
 /*
@@ -1118,8 +1111,9 @@ value omake_shell_sys_init(value v_unit)
  */
 value omake_shell_sys_close(value v_unit)
 {
+    CAMLparam1(v_unit);
     terminate_and_exit();
-    return Val_unit;
+    CAMLreturn(Val_unit);
 }
 
 #else /* WIN32 */
