@@ -48,7 +48,7 @@
  * ----------------------------------------------------------------
  *
  * @begin[license]
- * Copyright (C) 2003 Jason Hickey, Caltech
+ * Copyright (C) 2003-2006 Mojave Group, Caltech
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -64,8 +64,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * Author: Jason Hickey
- * @email{jyh@cs.caltech.edu}
+ * Author: Jason Hickey @email{jyh@cs.caltech.edu}
+ * Modified by: Aleksey Nogin @email{nogin@cs.caltech.edu}
  * @end[license]
  *)
 open Lm_printf
@@ -782,6 +782,7 @@ and pp_print_command_info_list buf infos =
 and pp_print_rule buf erule =
    let { rule_target      = target;
          rule_effects     = effects;
+         rule_locks       = locks;
          rule_sources     = sources;
          rule_scanners    = scanners;
          rule_commands    = commands
@@ -790,8 +791,9 @@ and pp_print_rule buf erule =
       fprintf buf "@[<hv 0>@[<hv 3>rule {";
       fprintf buf "@ target = %a" pp_print_node target;
       fprintf buf "@ @[<b 3>effects =%a@]" pp_print_node_set effects;
+      fprintf buf "@ @[<b 3>locks =%a@]" pp_print_node_set locks;
       fprintf buf "@ @[<b 3>sources =%a@]" pp_print_node_set sources;
-      fprintf buf "@ @[<b 3>sources =%a@]" pp_print_node_set scanners;
+      fprintf buf "@ @[<b 3>scanners =%a@]" pp_print_node_set scanners;
       fprintf buf "@ @[<hv 3>commands =%a@]" pp_print_command_info_list commands;
       fprintf buf "@]@ }@]"
 
@@ -1023,7 +1025,7 @@ struct
     * A .omc file.
     *)
    type t =
-      { db_file         : Unix.file_descr;
+      { db_file         : Unix.file_descr option;
         db_name         : Node.t;
         db_digest       : string;
         db_env          : venv;
@@ -1073,14 +1075,20 @@ struct
             name
       in
       let name = name ^ ".omc" in
-      let target_name, target_fd =
-         try Omake_state.lock_cache_file dir name with
+      let target_fd =
+         try 
+            let target_name, target_fd =
+               Omake_state.lock_cache_file dir name
+            in
+               if !debug_db then
+                  eprintf "@[<v 3>Omake_db.create:@ %a --> %s@]@." pp_print_node source target_name;
+               Some target_fd
+         with
             Unix.Unix_error _
           | Failure _ ->
-               raise Not_found
+               eprintf "@[<v 3>OMake warning: could not create and/or lock a cache file for@ %s@]@." source_name;
+               None
       in
-         if !debug_db then
-            eprintf "@[<v 3>Omake_db.create:@ %a --> %s@]@." pp_print_node source target_name;
          { db_file         = target_fd;
            db_name         = source;
            db_digest       = digest;
@@ -1092,108 +1100,88 @@ struct
    (*
     * Close the file.
     *)
-   let close info =
-      if !debug_db then
-         eprintf "Omake_db.close: %a@." pp_print_node info.db_name;
-      Unix.close info.db_file
+   let close = function
+      { db_file = Some fd; db_name = name } ->
+         if !debug_db then
+            eprintf "Omake_db.close: %a@." pp_print_node name;
+         Unix.close fd
+    | { db_file = None } ->
+         ()
 
    (*
     * Add the three kinds of entries.
     *)
    let add_ir info ir =
-      let { db_file   = fd;
-            db_name   = name;
-            db_digest = digest;
-            db_env    = venv
-          } = info
-      in
-         if !debug_db then
-            eprintf "Omake_db.add_ir: %a@." pp_print_node name;
-         venv_marshal venv (fun () ->
-               Lm_db.add fd (Node.absname name) ir_tag Omake_magic.ir_magic digest ir) ()
+      match info with
+         { db_file = Some fd; db_name = name; db_digest = digest; db_env = venv } ->
+            if !debug_db then
+               eprintf "Omake_db.add_ir: %a@." pp_print_node name;
+            venv_marshal venv (fun () ->
+                  Lm_db.add fd (Node.absname name) ir_tag Omake_magic.ir_magic digest ir) ()
+       | { db_file = None } ->
+            ()
 
    let add_object info obj =
-      let { db_file   = fd;
-            db_name   = name;
-            db_digest = digest;
-            db_env    = venv
-          } = info
-      in
-         if !debug_db then
-            eprintf "Omake_db.add_object: %a@." pp_print_node name;
-         venv_marshal venv (fun () ->
-               Lm_db.add fd (Node.absname name) object_tag Omake_magic.obj_magic digest obj) ()
+      match info with
+         { db_file = Some fd; db_name = name; db_digest = digest; db_env = venv } ->
+            if !debug_db then
+               eprintf "Omake_db.add_object: %a@." pp_print_node name;
+            venv_marshal venv (fun () ->
+                  Lm_db.add fd (Node.absname name) object_tag Omake_magic.obj_magic digest obj) ()
+       | { db_file = None } ->
+            ()
 
    let add_values info obj =
-      let { db_file   = fd;
-            db_name   = name;
-            db_digest = digest;
-            db_env    = venv
-          } = info
-      in
-         if !debug_db then
-            eprintf "Omake_db.add_values: %a@." pp_print_node name;
-         venv_marshal venv (fun () ->
-               Lm_db.add fd (Node.absname name) values_tag Omake_magic.obj_magic digest obj) ()
+      match info with
+         { db_file = Some fd; db_name = name; db_digest = digest; db_env = venv } ->
+            if !debug_db then
+               eprintf "Omake_db.add_values: %a@." pp_print_node name;
+            venv_marshal venv (fun () ->
+                  Lm_db.add fd (Node.absname name) values_tag Omake_magic.obj_magic digest obj) ()
+       | { db_file = None } ->
+            ()
 
    (*
     * Fetch the three kinds of entries.
     *)
-   let find_ir info =
-      let { db_file     = fd;
-            db_name     = name;
-            db_digest   = digest;
-            db_flush_ir = flush_ir
-          } = info
-      in
-      let () =
+   let find_ir = function
+      { db_file = Some fd; db_name = name; db_digest = digest; db_flush_ir = false } ->
          if !debug_db then
             eprintf "Omake_db.find_ir: finding: %a@." pp_print_node name;
-         if flush_ir then
-            raise Not_found
-      in
-      let ir = Lm_db.find fd (Node.absname name) ir_tag Omake_magic.ir_magic digest in
-         if !debug_db then
-            eprintf "Omake_db.find_ir: found: %a@." pp_print_node name;
-         ir
+         let ir = Lm_db.find fd (Node.absname name) ir_tag Omake_magic.ir_magic digest in
+            if !debug_db then
+               eprintf "Omake_db.find_ir: found: %a@." pp_print_node name;
+            ir
+    | { db_file = None }
+    | { db_flush_ir = true } ->
+         raise Not_found
 
-   let find_object info =
-      let { db_file         = fd;
-            db_name         = name;
-            db_digest       = digest;
-            db_flush_ir     = flush_ir;
-            db_flush_static = flush_static
-          } = info
-      in
-      let () =
+   let find_object = function
+      { db_file = Some fd; db_name = name; db_digest = digest; db_flush_ir = false; db_flush_static = false } ->
          if !debug_db then
             eprintf "Omake_db.find_object: finding: %a@." pp_print_node name;
-         if flush_ir || flush_static then
-            raise Not_found
-      in
-      let obj = Lm_db.find fd (Node.absname name) object_tag Omake_magic.obj_magic digest in
-         if !debug_db then
-            eprintf "Omake_db.find_object: found: %a@." pp_print_node name;
-         obj
+         let obj = Lm_db.find fd (Node.absname name) object_tag Omake_magic.obj_magic digest in
+            if !debug_db then
+               eprintf "Omake_db.find_object: found: %a@." pp_print_node name;
+            obj
+    | { db_file = None }
+    | { db_flush_ir = true }
+    | { db_flush_static = true } ->
+         raise Not_found
 
-   let find_values info =
-      let { db_file         = fd;
-            db_name         = name;
-            db_digest       = digest;
-            db_flush_ir     = flush_ir;
-            db_flush_static = flush_static
-          } = info
-      in
-      let () =
+   let find_values = function
+      { db_file = Some fd; db_name = name; db_digest = digest; db_flush_ir = false; db_flush_static = false } ->
          if !debug_db then
             eprintf "Omake_db.find_values: finding: %a@." pp_print_node name;
-         if flush_ir || flush_static then
-            raise Not_found
-      in
-      let obj = Lm_db.find fd (Node.absname name) values_tag Omake_magic.obj_magic digest in
-         if !debug_db then
-            eprintf "Omake_db.find_values: found: %a@." pp_print_node name;
-         obj
+         let obj = Lm_db.find fd (Node.absname name) values_tag Omake_magic.obj_magic digest in
+            if !debug_db then
+               eprintf "Omake_db.find_values: found: %a@." pp_print_node name;
+            obj
+    | { db_file = None }
+    | { db_flush_ir = true }
+    | { db_flush_static = true } ->
+         raise Not_found
+
 end;;
 
 (************************************************************************
@@ -2488,7 +2476,7 @@ let venv_add_explicit3_rules venv pos loc multiple targets locks patterns source
 
 (*
  * Figure out what to do based on all the parts.
- * A rule is implicit if it has not targets _or_ if the
+ * A rule is implicit if it has no targets _or_ if the
  * patterns do not contain a %.
  *)
 let venv_add_rule venv pos loc multiple targets patterns locks sources scanners values commands =
@@ -2537,6 +2525,10 @@ let venv_explicit_exists venv target =
    venv_explicit_flush venv;
    NodeSet.mem venv.venv_inner.venv_globals.venv_explicit_targets target
 
+let multiple_rules_err pos target buf =
+   fprintf buf "%a@ @[<hv3>Previous rule:@ %a@]" (**)
+      pp_print_exn (StringNodeError ("multiple ways to build", target)) pp_print_pos pos
+
 (*
  * Get the explicit rules.  Build a table indexed by target.
  *)
@@ -2558,7 +2550,8 @@ let venv_explicit_rules venv =
                      then
                         { erule with rule_commands = erule'.rule_commands @ erule.rule_commands }
                      else
-                        raise (OmakeException (loc_exp_pos erule.rule_loc, StringNodeError ("multiple ways to build", target)))
+                        raise (OmakeException (loc_exp_pos erule.rule_loc, (**)
+                           LazyError (multiple_rules_err (loc_exp_pos erule'.rule_loc) target)))
              | None ->
                   erule)
    in
@@ -2979,12 +2972,9 @@ let squash_prim_fun f =
 let squash_object obj =
    obj
 
-(*!
- * @docoff
- *
+(*
  * -*-
  * Local Variables:
- * Caml-master: "compile"
  * End:
  * -*-
  *)
