@@ -47,8 +47,7 @@ let tokens =
     "TokString"]
 
 let named_tokens =
-   ["TokComment", "{ string_white_exp $1 }";
-    "quote",      "{ $1 }";
+   ["quote",      "{ $1 }";
     "apply",      "{ $1 }"]
 
 let tokens =
@@ -72,8 +71,7 @@ let id =
     "TokClass"]
 
 let white =
-   ["TokWhite";
-    "TokComment"]
+   ["TokWhite"]
 
 let parens =
    ["TokLeftParen";
