@@ -236,6 +236,73 @@ let rec parse_ir venv scope node =
    let class_names, vars = senv_class_names senv in
       class_names, vars, ir
 
+(*
+ * When constructing a path, the relative filenames
+ * should be auto-rehash.
+ *
+ *    values  : the path
+ *    dirname : the subdirectory to search (often ".")
+ *)
+and path_of_values_select venv pos values dirname =
+   let rec collect groups auto_rehash items values =
+      match values with
+         v :: values ->
+            let rehash_flag, dir =
+               match v with
+                  ValDir dir ->
+                     false, dir
+                | ValNode node ->
+                     let dir = venv_intern_dir venv (string_of_value venv pos v) in
+                        false, dir
+                | _ ->
+                     let s = string_of_value venv pos v in
+                     let rehash_flag = not (Lm_filename_util.is_absolute s) in
+                     let dir = venv_intern_dir venv s in
+                        rehash_flag, dir
+            in
+            let dir = Dir.chdir dir dirname in
+            let groups, items =
+               if rehash_flag <> auto_rehash && items <> [] then
+                  (auto_rehash, List.rev items) :: groups, [dir]
+               else
+                  groups, dir :: items
+            in
+               collect groups rehash_flag items values
+       | [] ->
+            if items <> [] then
+               (auto_rehash, List.rev items) :: groups
+            else
+               groups
+   in
+      List.rev (collect [] false [] values)
+
+and path_of_values_rehash venv pos values dirname =
+   let dir_of_value v =
+      let dir =
+         match v with
+            ValDir dir ->
+               dir
+          | _ ->
+               venv_intern_dir venv (string_of_value venv pos v)
+      in
+         Dir.chdir dir dirname
+   in
+      [true, List.map dir_of_value values]
+
+and path_of_values venv pos values dirname =
+   let auto_rehash =
+      try bool_of_value venv pos (venv_find_var_exn venv ScopeGlobal auto_rehash_sym) with
+         Not_found ->
+            false
+   in
+   let f =
+      if auto_rehash then
+         path_of_values_rehash
+      else
+         path_of_values_select
+   in
+      f venv pos values dirname
+
 (*
  * Open the file.
  * Get the IR and return the vars.
@@ -263,19 +330,9 @@ and find_include_file venv pos loc filename =
          let dirname = Filename.dirname filename in
          let filename = Filename.basename filename in
          let fullname = filename ^ omake_file_suffix in
-         let dir_of_value v =
-            let dir =
-               match v with
-                  ValDir dir ->
-                     dir
-                | _ ->
-                     venv_intern_dir venv (string_of_value venv pos v)
-            in
-               Dir.chdir dir dirname
-         in
          let path = venv_find_var venv ScopeGlobal pos loc omakepath_sym in
          let path = values_of_value venv pos path in
-         let path = List.map dir_of_value path in
+         let path = path_of_values venv pos path dirname in
          let cache = venv_cache venv in
          let listing = Omake_cache.ls_path cache path in
             try
