@@ -258,6 +258,42 @@ let node_value_of_value venv pos v =
        | ValOther _ ->
             raise (OmakeException (pos, StringError "not a file"))
 
+let dir_value_of_value venv pos v =
+   let pos = string_pos "node_value_of_value" pos in
+   let arg = eval_prim_value venv pos v in
+      match arg with
+         ValNode _
+       | ValDir _ ->
+            arg
+       | ValData _
+       | ValQuote _
+       | ValString _
+       | ValSequence _
+       | ValArray _
+       | ValKey _
+       | ValApply _
+       | ValImplicit _
+       | ValSuperApply _
+       | ValMethodApply _
+       | ValBody _
+       | ValInt _
+       | ValFloat _ ->
+            let name = string_of_value venv pos v in
+               ValDir (venv_intern_dir venv name)
+       | ValNone
+       | ValEnv _
+       | ValFun _
+       | ValFunValue _
+       | ValPrim _
+       | ValRules _
+       | ValMap _
+       | ValObject _
+       | ValChannel _
+       | ValClass _
+       | ValCases _
+       | ValOther _ ->
+            raise (OmakeException (pos, StringError "not a file"))
+
 let filename_of_value venv pos v =
    let pos = string_pos "filename_of_value" pos in
    let arg = eval_prim_value venv pos v in
