@@ -68,7 +68,9 @@ type dir_entry_core =
    LazyEntryCore of Dir.t * string
  | DirEntryCore  of dir_entry
 
-type dir_listing = dir_entry_core ref StringTable.t
+type dir_listing_item = dir_entry_core ref StringTable.t
+
+type dir_listing = dir_listing_item list
 
 (*
  * Executable listing.
@@ -77,7 +79,9 @@ type exe_entry_core =
    ExeEntryCore of (Dir.t * string) list
  | ExeEntryNode of Node.t
 
-type exe_listing = exe_entry_core ref StringTable.t
+type exe_listing_item = exe_entry_core ref StringTable.t
+
+type exe_listing = exe_listing_item list
 
 (*
  * The memo is a record of what we have done during
@@ -109,8 +113,9 @@ type memo =
  *)
 (* %%MAGICBEGIN%% *)
 type node_stats =
-   FreshStats of Unix.LargeFile.stats
- | OldStats of Unix.LargeFile.stats
+   FreshStats   of Unix.LargeFile.stats
+ | OldStats     of Unix.LargeFile.stats
+ | PartialStats of Unix.LargeFile.stats
  | NoStats
 (* %%MAGICEND%% *)
 
@@ -138,6 +143,8 @@ type cache_info =
 (*
  * The cache remembers all the build commands we did last time.
  *)
+type stat = Unix.LargeFile.stats
+
 type cache =
    { (* State *)
      mutable cache_nodes             : node_memo NodeTable.t;
@@ -146,9 +153,9 @@ type cache =
      mutable cache_digest_count      : int;
 
      (* Path lookups *)
-     mutable cache_dirs         : dir_listing DirHashTable.t;
-     mutable cache_path         : dir_listing DirListTable.t;
-     mutable cache_exe_path     : exe_listing DirListTable.t;
+     mutable cache_dirs         : (stat option * dir_listing_item) DirHashTable.t;
+     mutable cache_path         : (stat option list * dir_listing_item) DirListTable.t;
+     mutable cache_exe_path     : (stat option list * exe_listing_item) DirListTable.t;
 
      (* Hash-cons versions of directories and paths *)
      cache_dir_hash             : DirHash.state;
@@ -304,6 +311,7 @@ let save_of_cache cache =
              | OldStats stats ->
                   let nmemo = { nmemo with nmemo_stats = OldStats stats } in
                      NodeTable.add nodes target nmemo
+             | PartialStats _
              | NoStats ->
                   nodes) NodeTable.empty cache_nodes
    in
@@ -389,6 +397,7 @@ let reset cache node =
                   FreshStats stats
                 | OldStats stats ->
                      Some { nmemo with nmemo_stats = OldStats stats }
+                | PartialStats _
                 | NoStats ->
                      None)
       with
@@ -408,15 +417,15 @@ let reset_table cache nodes =
  * file is up-to-date.
  *)
 let stats_equal stat1 stat2 =
-   let { Unix.LargeFile.st_ino  = ino1;
-         Unix.LargeFile.st_kind = kind1;
-         Unix.LargeFile.st_size = size1;
+   let { Unix.LargeFile.st_ino   = ino1;
+         Unix.LargeFile.st_kind  = kind1;
+         Unix.LargeFile.st_size  = size1;
          Unix.LargeFile.st_mtime = mtime1
        } = stat1
    in
-   let { Unix.LargeFile.st_ino  = ino2;
-         Unix.LargeFile.st_kind = kind2;
-         Unix.LargeFile.st_size = size2;
+   let { Unix.LargeFile.st_ino   = ino2;
+         Unix.LargeFile.st_kind  = kind2;
+         Unix.LargeFile.st_size  = size2;
          Unix.LargeFile.st_mtime = mtime2
        } = stat2
    in
@@ -487,6 +496,34 @@ let stat_file cache node =
                cache.cache_nodes <- NodeTable.add nodes node nmemo;
                nmemo.nmemo_digest
 
+       | Some { nmemo_stats = PartialStats stats } ->
+            (*
+             * We have taken stats of this file, and found that they
+             * differed from the old value, so the digest is out-of-date.
+             *)
+            let name = Node.fullname node in
+            let nmemo =
+               try
+                  if stats.Unix.LargeFile.st_kind = Unix.S_DIR then
+                     { nmemo_stats = FreshStats stats;
+                       nmemo_digest = squash_stat
+                     }
+                  else
+                     let digest = Digest.file name in
+                        cache.cache_digest_count <- succ cache.cache_digest_count;
+                        { nmemo_stats = FreshStats stats;
+                          nmemo_digest = Some digest
+                        }
+               with
+                  Unix.Unix_error _
+                | Sys_error _ ->
+                     { nmemo_stats = NoStats;
+                       nmemo_digest = None
+                     }
+            in
+               cache.cache_nodes <- NodeTable.add nodes node nmemo;
+               nmemo.nmemo_digest
+
        | None ->
             (*
              * We've never seen this file before.
@@ -500,13 +537,14 @@ let stat_file cache node =
                         { nmemo_stats = FreshStats stats;
                           nmemo_digest = squash_stat
                         }
-                     else
+                     else begin
+                        cache.cache_file_stat_count <- succ cache.cache_file_stat_count;
                         let digest = Digest.file name in
-                           cache.cache_file_stat_count <- succ cache.cache_file_stat_count;
                            cache.cache_digest_count <- succ cache.cache_digest_count;
                            { nmemo_stats = FreshStats stats;
                              nmemo_digest = Some digest
                            }
+                     end
                with
                   Unix.Unix_error _
                 | Sys_error _ ->
@@ -518,22 +556,81 @@ let stat_file cache node =
                nmemo.nmemo_digest
 
 (*
- * Return stat info.
+ * Return just the Unix stat information.
+ * Avoid taking a full digest in this case.
  *)
-let rec stat_unix cache node =
+let stat_unix_simple stats =
+   match stats with
+      FreshStats stats
+    | PartialStats stats ->
+         stats
+    | NoStats ->
+         raise Not_found
+    | OldStats _ ->
+         raise (Invalid_argument "Omake_cache.stat_unix_simple")
+
+let stat_unix cache node =
+   let nodes = cache.cache_nodes in
    let stats =
-      try Some (NodeTable.find cache.cache_nodes node).nmemo_stats with
+      try Some (NodeTable.find nodes node) with
          Not_found ->
             None
    in
       match stats with
-         Some (FreshStats stats) ->
+         Some { nmemo_stats = FreshStats stats }
+       | Some { nmemo_stats = PartialStats stats } ->
             stats
-       | Some NoStats ->
+       | Some { nmemo_stats = NoStats } ->
             raise Not_found
-       | _ ->
-            let _ = stat_file cache node in
-               stat_unix cache node
+       | Some ({ nmemo_stats = OldStats old_stats } as nmemo) ->
+            let name = Node.fullname node in
+            let nmemo =
+               try
+                  let new_stats = Unix.LargeFile.stat name in
+                     if new_stats.Unix.LargeFile.st_kind = Unix.S_DIR then
+                        { nmemo_stats = FreshStats new_stats;
+                          nmemo_digest = squash_stat
+                        }
+                     else begin
+                        cache.cache_file_stat_count <- succ cache.cache_file_stat_count;
+                        if stats_equal new_stats old_stats then
+                           { nmemo with nmemo_stats = FreshStats new_stats }
+                        else
+                           { nmemo with nmemo_stats = PartialStats new_stats }
+                     end
+               with
+                  Unix.Unix_error _
+                | Sys_error _ ->
+                     { nmemo_stats = NoStats;
+                       nmemo_digest = None
+                     }
+            in
+               cache.cache_nodes <- NodeTable.add nodes node nmemo;
+               stat_unix_simple nmemo.nmemo_stats
+       | None ->
+            let name = Node.fullname node in
+            let nmemo =
+               try
+                  let stats = Unix.LargeFile.stat name in
+                     if stats.Unix.LargeFile.st_kind = Unix.S_DIR then
+                        { nmemo_stats = FreshStats stats;
+                          nmemo_digest = squash_stat
+                        }
+                     else begin
+                        cache.cache_file_stat_count <- succ cache.cache_file_stat_count;
+                        { nmemo_stats = PartialStats stats;
+                          nmemo_digest = None
+                        }
+                     end
+               with
+                  Unix.Unix_error _
+                | Sys_error _ ->
+                     { nmemo_stats  = NoStats;
+                       nmemo_digest = None
+                     }
+            in
+               cache.cache_nodes <- NodeTable.add nodes node nmemo;
+               stat_unix_simple nmemo.nmemo_stats
 
 (*
  * Check if a file is a directory.
@@ -557,23 +654,6 @@ let groups =
       Unix.Unix_error _ ->
          []
 
-let is_exe_file cache node =
-   try
-      let { Unix.LargeFile.st_kind = kind;
-            Unix.LargeFile.st_perm = perm;
-            Unix.LargeFile.st_uid = uid;
-            Unix.LargeFile.st_gid = gid
-          } = stat_unix cache node
-      in
-         (kind = Unix.S_REG)
-         && ((perm land 0o001) <> 0
-             || (List.mem gid groups && (perm land 0o010) <> 0)
-             || (uid = euid && (perm land 0o100) <> 0))
-   with
-      Unix.Unix_error _
-    | Not_found ->
-         false
-
 (*
  * Check if the node is phony first.
  *)
@@ -822,6 +902,55 @@ let find_result_sloppy cache key target =
  * Directory listings.
  *)
 
+(*
+ * When auto-rehash is in effect, we need to stat the directories
+ * on every lookup.
+ *)
+let stat_dir cache dir =
+   let name = Dir.fullname dir in
+      try
+         let stat = Unix.LargeFile.stat name in
+            cache.cache_file_stat_count <- succ cache.cache_file_stat_count;
+            Some stat
+      with
+         Unix.Unix_error _ ->
+            None
+
+let stat_dirs cache dirs =
+   List.map (stat_dir cache) dirs
+
+let stats_equal_opt stat1 stat2 =
+   match stat1, stat2 with
+      Some stat1, Some stat2 ->
+         stats_equal stat1 stat2
+    | None, None ->
+         true
+    | None, Some _
+    | Some _, None ->
+         false
+
+let rec stats_equal_opt_list stats1 stats2 =
+   match stats1, stats2 with
+      stat1 :: stats1, stat2 :: stats2 ->
+         stats_equal_opt stat1 stat2 && stats_equal_opt_list stats1 stats2
+    | [], [] ->
+         true
+    | _ :: _, []
+    | [], _ :: _ ->
+         false
+
+let check_stat auto_rehash (stat_old, entries) stat_new =
+   if auto_rehash && not (stats_equal_opt stat_old (Lazy.force stat_new)) then
+      raise Not_found
+   else
+      entries
+
+let check_stats auto_rehash (stats_old, entries) stats_new =
+   if auto_rehash && not (stats_equal_opt_list stats_old (Lazy.force stats_new)) then
+      raise Not_found
+   else
+      entries
+
 (*
  * List a directory.
  *)
@@ -856,39 +985,43 @@ let rec list_directory cache dir =
 (*
  * Get the directory listing as a StringTable.
  *)
-let ls_dir cache dir =
+let ls_dir cache auto_rehash dir =
    let key = DirHash.create cache.cache_dir_hash dir in
-      try DirHashTable.find cache.cache_dirs key with
+   let stat = lazy (stat_dir cache dir) in
+      try check_stat auto_rehash (DirHashTable.find cache.cache_dirs key) stat with
          Not_found ->
             let entries = list_directory cache dir in
-               cache.cache_dirs <- DirHashTable.add cache.cache_dirs key entries;
+            let stat = Lazy.force stat in
+               cache.cache_dirs <- DirHashTable.add cache.cache_dirs key (stat, entries);
                entries
 
 (*
  * Path version.
  *)
-let ls_path cache dirs =
+let ls_path cache auto_rehash dirs =
    let keys = List.map (DirHash.create cache.cache_dir_hash) dirs in
    let key = DirListHash.create cache.cache_path_hash keys in
-      try DirListTable.find cache.cache_path key with
+   let stats = lazy (stat_dirs cache dirs) in
+      try check_stats auto_rehash (DirListTable.find cache.cache_path key) stats with
          Not_found ->
             (* Fold together the tables *)
+            let stats = Lazy.force stats in
             let entries =
                List.fold_left (fun entries1 dir ->
                      try
-                        let entries2 = ls_dir cache dir in
+                        let entries2 = ls_dir cache auto_rehash dir in
                            StringTable.fold StringTable.add entries1 entries2
                      with
                         Not_found ->
                            entries1) StringTable.empty (List.rev dirs)
             in
-               cache.cache_path <- DirListTable.add cache.cache_path key entries;
+               cache.cache_path <- DirListTable.add cache.cache_path key (stats, entries);
                entries
 
 (*
  * Resolve an entry in the listing.
  *)
-let listing_find cache listing s =
+let listing_find_item cache listing s =
    let entry_ref = StringTable.find listing s in
       match !entry_ref with
          DirEntryCore entry ->
@@ -920,15 +1053,16 @@ let listing_find cache listing s =
  *    - There is no .exe suffix
  *    - Only files that are executable count
  *)
-let ls_exe_path_win32 cache dirs =
+let ls_exe_path_win32 cache auto_rehash dirs =
    let keys = List.map (DirHash.create cache.cache_dir_hash) dirs in
    let key = DirListHash.create cache.cache_path_hash keys in
-      try DirListTable.find cache.cache_exe_path key with
+   let stats = lazy (stat_dirs cache dirs) in
+      try check_stats auto_rehash (DirListTable.find cache.cache_exe_path key) stats with
          Not_found ->
             let entries =
                List.fold_left (fun entries1 dir ->
                      try
-                        let entries2 = ls_dir cache dir in
+                        let entries2 = ls_dir cache auto_rehash dir in
                            StringTable.fold (fun entries name _ ->
                                  let info = dir, name in
                                  let name = String.lowercase name in
@@ -946,18 +1080,20 @@ let ls_exe_path_win32 cache dirs =
                StringMTable.fold_all (fun entries name info ->
                      StringTable.add entries name (ref (ExeEntryCore info))) StringTable.empty entries
             in
-               cache.cache_exe_path <- DirListTable.add cache.cache_exe_path key entries;
+            let stats = Lazy.force stats in
+               cache.cache_exe_path <- DirListTable.add cache.cache_exe_path key (stats, entries);
                entries
 
-let ls_exe_path_unix cache dirs =
+let ls_exe_path_unix cache auto_rehash dirs =
    let keys = List.map (DirHash.create cache.cache_dir_hash) dirs in
    let key = DirListHash.create cache.cache_path_hash keys in
-      try DirListTable.find cache.cache_exe_path key with
+   let stats = lazy (stat_dirs cache dirs) in
+      try check_stats auto_rehash (DirListTable.find cache.cache_exe_path key) stats with
          Not_found ->
             let entries =
                List.fold_left (fun entries1 dir ->
                      try
-                        let entries2 = ls_dir cache dir in
+                        let entries2 = ls_dir cache auto_rehash dir in
                            StringTable.fold (fun entries name _ ->
                                  StringMTable.add entries name (dir, name)) entries1 entries2
                      with
@@ -968,12 +1104,30 @@ let ls_exe_path_unix cache dirs =
                StringMTable.fold_all (fun entries name info ->
                      StringTable.add entries name (ref (ExeEntryCore info))) StringTable.empty entries
             in
-               cache.cache_exe_path <- DirListTable.add cache.cache_exe_path key entries;
+            let stats = Lazy.force stats in
+               cache.cache_exe_path <- DirListTable.add cache.cache_exe_path key (stats, entries);
                entries
 
 (*
  * Find the first entry that is executable.
  *)
+let is_exe_file cache node =
+   try
+      let { Unix.LargeFile.st_kind = kind;
+            Unix.LargeFile.st_perm = perm;
+            Unix.LargeFile.st_uid = uid;
+            Unix.LargeFile.st_gid = gid
+          } = Unix.LargeFile.stat (Node.fullname node)
+      in
+         (kind = Unix.S_REG)
+         && ((perm land 0o001) <> 0
+             || (List.mem gid groups && (perm land 0o010) <> 0)
+             || (uid = euid && (perm land 0o100) <> 0))
+   with
+      Unix.Unix_error _
+    | Not_found ->
+         false
+
 let rec search_exe_win32 cache entries =
    match entries with
       (dir, s) :: entries ->
@@ -1015,7 +1169,7 @@ let ls_exe_path, search_exe, name_exe =
    else
       ls_exe_path_unix, search_exe_unix, (fun s -> s)
 
-let exe_find cache listing s =
+let exe_find_item cache listing s =
    let entry_ref = StringTable.find listing (name_exe s) in
       match !entry_ref with
          ExeEntryCore entries ->
@@ -1025,6 +1179,43 @@ let exe_find cache listing s =
        | ExeEntryNode node ->
             node
 
+(************************************************************************
+ * Redefine the functions to work on directory groups, where each group may
+ * specify auto-rehashing.
+ *)
+let rec listing_find cache listings s =
+   match listings with
+      [listing] ->
+         listing_find_item cache listing s
+    | listing :: listings ->
+         (try listing_find_item cache listing s with
+             Not_found ->
+                listing_find cache listings s)
+    | [] ->
+         raise Not_found
+
+let rec exe_find cache listings s =
+   match listings with
+      [listing] ->
+         exe_find_item cache listing s
+    | listing :: listings ->
+         (try exe_find_item cache listing s with
+             Not_found ->
+                exe_find cache listings s)
+    | [] ->
+         raise Not_found
+
+let ls_dir cache auto_rehash dir =
+   [ls_dir cache auto_rehash dir]
+
+let ls_path cache groups =
+   List.map (fun (auto_rehash, dirs) ->
+         ls_path cache auto_rehash dirs) groups
+
+let ls_exe_path cache groups =
+   List.map (fun (auto_rehash, dirs) ->
+         ls_exe_path cache auto_rehash dirs) groups
+
 (*!
  * @docoff
  *
