@@ -76,8 +76,10 @@ let install_current force =
 (*
  * Install into all subdirectories.
  *)
+let glob_cvs_ignore = Lm_glob.create_options [GlobCVSIgnore]
+
 let install_subdirs force =
-   let dirs = subdirs_of_dirs [GlobCVSIgnore] "" ["."] in
+   let dirs = subdirs_of_dirs glob_cvs_ignore "" ["."] in
       copy_file force omakeroot "OMakeroot";
       List.iter (fun dir -> copy_file force omakefile (Filename.concat dir "OMakefile")) dirs;
       printf "*** omake: project files OMakefile and OMakeroot have been installed\n";
