@@ -1059,7 +1059,8 @@ struct
       (* Parse the .PHONY syntax *)
       let name = Lm_filename_util.unescape_string name in
          match parse_phony_name name, phony_ok with
-            PhonyDirString name, (PhonyOK | PhonyExplicit) ->
+            PhonyDirString name, PhonyOK
+          | PhonyDirString name, PhonyExplicit ->
                (* Name is .PHONY/... *)
                let hash, dir, key, name = new_file dir name in
                let node = NodePhonyDir (hash, dir, key, name) in
@@ -1068,7 +1069,8 @@ struct
                          base.base_nodes <- NodeTable.add nodes node node;
                          node)
 
-          | PhonyGlobalString name, (PhonyOK | PhonyExplicit) ->
+          | PhonyGlobalString name, PhonyOK
+          | PhonyGlobalString name, PhonyExplicit ->
                (* Name is /.PHONY/name *)
                let hash = Hash.hash_global_phony name in
                let node = NodePhonyGlobal (hash, name) in
@@ -1077,7 +1079,8 @@ struct
                          base.base_nodes <- NodeTable.add nodes node node;
                          node)
 
-          | (PhonyDirString _| PhonyGlobalString _), PhonyProhibited ->
+          | PhonyDirString _, PhonyProhibited
+          | PhonyGlobalString _, PhonyProhibited ->
                raise (Invalid_argument "Omake_node.Node.intern: NodePhony is not allowed");
 
           | PhonySimpleString, PhonyOK ->
@@ -1099,7 +1102,8 @@ struct
                                            base.base_nodes <- NodeTable.add nodes node node;
                                            node)
 
-          | PhonySimpleString, (PhonyExplicit | PhonyProhibited)
+          | PhonySimpleString, PhonyExplicit
+          | PhonySimpleString, PhonyProhibited
           | PhonyPathString, _ ->
                (* Otherwise, it is a normal file *)
                let hash, dir, key, name = new_file dir name in
@@ -1109,16 +1113,6 @@ struct
                         base.base_nodes <- NodeTable.add nodes node node;
                         node
 
-   (*
-    * Core node without modifiers.
-    *)
-   let core node =
-      match node with
-         NodeFlagged (_, node) ->
-            node
-       | _ ->
-            node
-
    (*
     * Escape a node.
     *)
@@ -1169,6 +1163,24 @@ struct
        | NodeFlagged (_, node) ->
             core node
 
+   (*
+    * For building targets, we sometimes want to know the
+    * original node.
+    *)
+   let rec unsquash node =
+      match node with
+         NodePhonyGlobal _
+       | NodePhonyDir _
+       | NodePhonyFile _
+       | NodeFile _
+       | NodeFlagged (NodeIsOptional, _)
+       | NodeFlagged (NodeIsScanner, _)
+       | NodeFlagged (NodeIsExisting, _) ->
+            node
+
+       | NodeFlagged (NodeIsSquashed, node) ->
+            unsquash node
+
    (*
     * Kind of the node.
     *)
