@@ -141,9 +141,9 @@ type node_kind =
  | NodeNormal
 
 type phony_ok =
-   PhonyOK (* It is OK to return a phony node *)
- | PhonyExplicit (* It is OK to return a phony node only when an explicit syntax was used *)
- | PhonyProhibited (* Returning phony nodes is not allowed *)
+   PhonyOK             (* It is OK to return a phony node *)
+ | PhonyExplicit       (* It is OK to return a phony node only when an explicit syntax was used *)
+ | PhonyProhibited     (* Returning phony nodes is not allowed *)
 
 (*
  * A file node.  There are two kinds of nodes.
@@ -185,6 +185,11 @@ sig
     *)
    val core : t -> t
 
+   (*
+    * Get the core node if the node is squashed.
+    *)
+   val unsquash : t -> t
+
    (*
     * Does the node always exist?
     *)
@@ -241,11 +246,6 @@ sig
    val compare : t -> t -> int
    val compare_alpha : t -> t -> int
 
-   (*
-    * The core node.
-    *)
-   val core : t -> t
-
    (*
     * Just the tail.
     *)
