@@ -177,7 +177,7 @@ let spec =
            "-debug-db", Lm_arg.Set Omake_env.debug_db, (**)
               "Debug the file database";
            "-allow-exceptions", Lm_arg.UnitFold (fun options -> { options with opt_allow_exceptions = true }),
-              "Do not catch top-level exceptions (for use with OCAMRUNPARAM=b)"];
+              "Do not catch top-level exceptions (for use with OCAMLRUNPARAM=b)"];
        "Internal flags", (**)
           ["-server", Lm_arg.String (fun s -> server_flag := Some s), (**)
               "Run as a remote server";]]
