@@ -51,39 +51,6 @@ open Pos;;
  *)
 let node_empty = Node.phony_global "interactive shell input"
 
-(*
- * Cache operations.
- *)
-let load_cache () =
-   try
-      let inx = open_in_bin (db_file ()) in
-         try
-            let cache = Omake_cache.from_channel inx in
-               close_in inx;
-               cache
-         with
-            Failure _
-          | End_of_file
-          | Sys_error _ ->
-               close_in inx;
-               Omake_cache.create ()
-   with
-      Sys_error _ ->
-         Omake_cache.create ()
-
-let save_cache venv =
-   let cache = venv_cache venv in
-   let db_file = db_file () in
-      try
-         let outx = Pervasives.open_out_bin db_file in
-            try Omake_cache.to_channel outx cache with
-               exn ->
-                  close_out outx;
-                  raise exn
-      with
-         Sys_error _ ->
-            eprintf "*** omake: can't save cache file: %s@." db_file
-
 (*
  * The parser.
  *)
@@ -278,10 +245,8 @@ let rec main state senv venv result =
                eprintf "^D@.Use \"exit\" leave osh.@.";
                senv, venv, result
             end
-            else begin
-               save_cache venv;
+            else
                exit 0
-            end
        | Unix.Unix_error _
        | Invalid_argument _
        | Sys_error _
@@ -400,7 +365,7 @@ let create_venv options targets =
 
    (* Now start creating *)
    let exec  = Exec.create cwd options in
-   let cache = load_cache () in
+   let cache = Omake_cache.create () in
    let venv  = Omake_env.create options "." exec cache in
    let venv  = venv_chdir_tmp venv cwd in
    let venv  = Omake_builtin.venv_add_command_defs venv in
