@@ -605,7 +605,7 @@ let find_executable venv pos loc exe =
                raise (OmakeException (loc_pos loc pos, StringStringError ("command not found", exe)))
       else
          let path = venv_find_var venv ScopeGlobal pos loc path_sym in
-         let path = List.map (dir_of_value venv pos) (values_of_value venv pos path) in
+         let path = Omake_eval.path_of_values venv pos (values_of_value venv pos path) "." in
          let path = Omake_cache.ls_exe_path cache path in
             try Omake_cache.exe_find cache path exe with
                Not_found ->
