@@ -133,23 +133,6 @@ let create_thread info =
  *)
 let create_process_group = create_thread
 
-(*
- * Check signals in the blocking_section function.
- *)
-let blocking_section f x =
-   if check_thread () then
-      raise Terminated;
-   let y =
-      try Lm_thread_pool.blocking_section f x with
-         exn ->
-            if check_thread () then
-               raise Terminated;
-            raise exn
-   in
-      if check_thread () then
-         raise Terminated;
-      y
-
 (*!
  * @docoff
  *
